using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Printing;
using System.Data.OleDb;
using System.IO;

namespace Pizza_Order
{
  public partial class frmPizza : Form
  {
    public frmPizza()
    {
      InitializeComponent();
    }

    int orderNumber;
    OleDbConnection pizzaConnection;
    OleDbCommand customersCommand;
    OleDbDataAdapter customersAdapter;
    DataTable customersTable;
    CurrencyManager customersManager;
    OleDbCommand ordersCommand;
    OleDbDataAdapter ordersAdapter;
    DataTable ordersTable;
    OleDbCommand pizzasCommand;
    OleDbDataAdapter pizzasAdapter;
    DataTable pizzasTable;
    int pizzaSize, numberPizzas;
    float totalCost;
    float[] toppingCost = new float[12];
    float[] sizeCost = new float[3];
    float deliveryCost;
    float taxRate;
    const int numberToppings = 12;
    CheckBox[] chkTop = new CheckBox[12];
    RadioButton[] rdoSize = new RadioButton[3];

    private void frmPizza_Load(object sender, EventArgs e)
    {
      // Topping cost information (veggies 50 cents, meats 1 dollar)
      chkTop[0] = chkCheese; toppingCost[0] = 0.5F;
      chkTop[1] = chkMushrooms; toppingCost[1] = 0.5F;
      chkTop[2] = chkBlackOlives; toppingCost[2] = 0.5F;
      chkTop[3] = chkOnions; toppingCost[3] = 0.5F;
      chkTop[4] = chkGreenPeppers; toppingCost[4] = 0.5F;
      chkTop[5] = chkTomatoes; toppingCost[5] = 0.5F;
      chkTop[6] = chkPepperoni; toppingCost[6] = 1.0F;
      chkTop[7] = chkSalami; toppingCost[7] = 1.0F;
      chkTop[8] = chkSausage; toppingCost[8] = 1.0F;
      chkTop[9] = chkGroundBeef; toppingCost[9] = 1.0F;
      chkTop[10] = chkShrimp; toppingCost[10] = 1.0F;
      chkTop[11] = chkAnchovies; toppingCost[11] = 1.0F;
      // Size cost
      rdoSize[0] = rdoSmall; sizeCost[0] = 4.0F;
      rdoSize[1] = rdoMedium; sizeCost[1] = 6.0F;
      rdoSize[2] = rdoLarge; sizeCost[2] = 8.0F;
      pizzaSize = 0;
      // Delivery cost
      deliveryCost = 1.5F;
      // Tax rate
      taxRate = 8.9F;
      // get order number
      StreamReader inputFile = new StreamReader(Application.StartupPath + "\\..\\..\\pizza.ini");
      orderNumber = Convert.ToInt32(inputFile.ReadLine());
      inputFile.Close();
      this.Text = "Pizza Order #" + orderNumber.ToString().PadLeft(4, '0');
      // connect to pizza database
      pizzaConnection = new OleDbConnection("Provider=Microsoft.Jet.OLEDB.4.0; Data Source = " + Application.StartupPath + "\\..\\..\\PizzaDB.mdb");
      pizzaConnection.Open();
      // establish Customers command object
      customersCommand = new OleDbCommand("SELECT * FROM Customers ORDER BY PhoneNumber", pizzaConnection);
      // establish Customers data adapter/data table
      customersAdapter = new OleDbDataAdapter();
      customersAdapter.SelectCommand = customersCommand;
      customersTable = new DataTable();
      customersAdapter.Fill(customersTable);
      // bind controls to data table
      txtFirstName.DataBindings.Add("Text", customersTable, "FirstName");
      txtLastName.DataBindings.Add("Text", customersTable, "LastName");
      txtAddress.DataBindings.Add("Text", customersTable, "Address");
      txtCity.DataBindings.Add("Text", customersTable, "City");
      txtState.DataBindings.Add("Text", customersTable, "State");
      txtZip.DataBindings.Add("Text", customersTable, "Zip");
      txtComment.DataBindings.Add("Text", customersTable, "Comment");
      // establish currency manager
      customersManager = (CurrencyManager) this.BindingContext[customersTable];
      // establish Orders command object
      ordersCommand = new OleDbCommand("SELECT * FROM Orders ORDER BY PhoneNumber", pizzaConnection);
      // establish Orders data adapter/data table
      ordersAdapter = new OleDbDataAdapter();
      ordersAdapter.SelectCommand = ordersCommand;
      ordersTable = new DataTable();
      ordersAdapter.Fill(ordersTable);
      // establish Pizzas command object
      pizzasCommand = new OleDbCommand("SELECT * FROM Pizzas ORDER BY OrderNumber", pizzaConnection);
      // establish Pizzas data adapter/data table
      pizzasAdapter = new OleDbDataAdapter();
      pizzasAdapter.SelectCommand = pizzasCommand;
      pizzasTable = new DataTable();
      pizzasAdapter.Fill(pizzasTable);
      lblDate.Text = DateTime.Now.ToShortDateString();
      txtPhone.Focus();
    }

    private void frmPizza_FormClosing(object sender, FormClosingEventArgs e)
    {
      // save order number
      StreamWriter outputFile = new StreamWriter(Application.StartupPath + "\\..\\..\\pizza.ini");
      outputFile.WriteLine(orderNumber);
      outputFile.Close();
      try
      {
        // save the tables to the database file
        OleDbCommandBuilder ordersAdapterCommands = new OleDbCommandBuilder(ordersAdapter);
        ordersAdapter.Update(ordersTable);
        OleDbCommandBuilder customersAdapterCommands = new OleDbCommandBuilder(customersAdapter);
        customersAdapter.Update(customersTable);
        OleDbCommandBuilder pizzasAdapterCommands = new OleDbCommandBuilder(pizzasAdapter);
        pizzasAdapter.Update(pizzasTable);
      }
      catch (Exception ex)
      {
        MessageBox.Show("Error saving database", "Save Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
      // close the connection
      pizzaConnection.Close();
      // dispose of the objects
      ordersCommand.Dispose();
      ordersAdapter.Dispose();
      ordersTable.Dispose();
      customersCommand.Dispose();
      customersAdapter.Dispose();
      customersTable.Dispose();
      pizzasCommand.Dispose();
      pizzasAdapter.Dispose();
      pizzasTable.Dispose();
    }

    private void btnExit_Click(object sender, EventArgs e)
    {
      this.Close();
    }

    private void btnFind_Click(object sender, EventArgs e)
    {
      // Check phone number validity (at least that it has enough characters)
      int l = txtPhone.Text.Length;
      if (l != 14)
      {
        MessageBox.Show("Phone number requires 10 digits.", "Phone Number Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        txtPhone.Focus();
        return;
      }
      customersTable.DefaultView.Sort = "PhoneNumber";
      int foundRow = customersTable.DefaultView.Find(txtPhone.Text);
      if (foundRow != -1)
      {
        btnFind.Enabled = false;
        grpSize.Enabled = true;
        grpDeliver.Enabled = true;
        grpToppings.Enabled = true;
        txtSpecial.Enabled = true;
        btnBuild.Enabled = true;
        btnComplete.Enabled = false;
        txtPhone.Enabled = false;
        txtFirstName.Visible = true;
        txtLastName.Visible = true;
        txtAddress.Visible = true;
        txtCity.Visible = true;
        txtState.Visible = true;
        txtZip.Visible = true;
        txtComment.Visible = true;
        totalCost = 0;
        numberPizzas = 0;
        customersManager.Position = foundRow;
      }
      else
      {
        // code for new customer
        btnFind.Enabled = false;
        txtPhone.Enabled = false;
        txtFirstName.Visible = true;
        txtLastName.Visible = true;
        txtAddress.Visible = true;
        txtCity.Visible = true;
        txtState.Visible = true;
        txtZip.Visible = true;
        txtComment.Visible = true;
        customersManager.AddNew();
        txtFirstName.Focus();
        btnSave.Visible = true;
        btnCancel.Visible = true;
      }
    }

    private void btnBuild_Click(object sender, EventArgs e)
    {
      // This procedure builds a message box that displays your pizza type
      string message = "", toppingString;
      float cost;
      if (pizzaSize < 0)
      {
        MessageBox.Show("You must choose a size", "Error", MessageBoxButtons.OK, MessageBoxIcon.Information);
        return;
      }
      message += rdoSize[pizzaSize].Text + " pizza\r\n";
      cost = sizeCost[pizzaSize];
      toppingString = "";
      for (int i = 0; i < numberToppings; i++)
      {
        if (chkTop[i].Checked)
        {
          message += chkTop[i].Text + "\r\n";
          toppingString += "1";
          cost += toppingCost[i];
        }
        else
        {
          toppingString += "0";
        }
      }
      if (toppingString.Equals("000000000000"))
        message += "Cheese only\r\n";
      if (!txtSpecial.Text.Equals(""))
        message += txtSpecial.Text + "\r\n";
      if (chkDeliver.Checked)
      {
        message += "To be delivered\r\n";
        cost += deliveryCost;
      }
      else
      {
        message += "For pickup\r\n";
      }
      message += "Cost is $" + String.Format("{0:f2}", cost);
      // if ok add to database and order; if not just exit
      if (MessageBox.Show(message + "\r\n\r\n" + "Is this correct?", "Your Pizza", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
      {
        // Add pizza to pizzas table
        DataRow newRow = pizzasTable.NewRow();
        newRow["OrderNumber"] = orderNumber;
        if (chkDeliver.Checked)
          newRow["Delivery"] = "1";
        else
          newRow["Delivery"] = "0";
        newRow["PizzaSize"] = pizzaSize.ToString();
        newRow["Toppings"] = toppingString;
        newRow["SpecialInfo"] = txtSpecial.Text;
        newRow["Cost"] = cost;
        pizzasTable.Rows.Add(newRow);
        numberPizzas++;
        totalCost += cost;
        cost = 0;
        pizzaSize = 0;
        rdoSize[pizzaSize].Checked = true;
        for (int i = 0; i < numberToppings; i++)
          chkTop[i].Checked = false;
        txtSpecial.Text = "";
        chkDeliver.Checked = false;
        btnComplete.Enabled = true;
      }
    }

    private void btnComplete_Click(object sender, EventArgs e)
    {
      totalCost = (1 + taxRate / 100) * totalCost;
      if (MessageBox.Show("Order includes " + numberPizzas.ToString() + " pizza(s)\r\nTotal cost is $" + String.Format("{0:f2}", totalCost) + " (including tax)\r\n\r\nWould you like a printed receipt?", "Order Complete", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
        PrintReceipt();
      // Write order number to database - get ready for new customer
      DataRow newRow = ordersTable.NewRow();
      newRow["OrderNumber"] = orderNumber;
      newRow["PhoneNumber"] = txtPhone.Text;
      newRow["OrderDate"] = lblDate.Text;
      newRow["TotalCost"] = totalCost;
      ordersTable.Rows.Add(newRow);
      orderNumber++;
      this.Text = "Pizza Order #" + orderNumber.ToString().PadLeft(4, '0');
      btnFind.Enabled = true;
      grpSize.Enabled = false;
      grpDeliver.Enabled = false;
      grpToppings.Enabled = false;
      txtSpecial.Enabled = false;
      btnBuild.Enabled = false;
      btnComplete.Enabled = false;
      txtFirstName.Visible = false;
      txtLastName.Visible = false;
      txtAddress.Visible = false;
      txtCity.Visible = false;
      txtState.Visible = false;
      txtZip.Visible = false;
      txtComment.Visible = false;
      txtPhone.Enabled = true;
      txtPhone.Text = "";
      txtPhone.Focus();
    }

    private void UpdateTitle()
    {
      string o = orderNumber.ToString();
      if (orderNumber < 10)
        o = "000" + o;
      else if (orderNumber < 100)
        o = "00" + o;
      else if (orderNumber < 1000)
        o = "0" + o;
      this.Text = "Pizza Order #" + o;
    }

    private void PrintReceipt()
    {
      PrintDocument recordDocument;
      // Create the document and name it
      recordDocument = new PrintDocument();
      recordDocument.DocumentName = "Pizza Receipt";
      recordDocument.PrintPage += new PrintPageEventHandler(this.PrintReceiptPage);
      // Preview document
      dlgPreview.Document = recordDocument;
      dlgPreview.ShowDialog();
      // Dispose of document when done printing
      recordDocument.Dispose();
    }


    private void PrintReceiptPage(object sender, PrintPageEventArgs e)
    {
      string s;
      int y;
      // Print receipt
      Font myFont = new Font("Arial", 12, FontStyle.Bold);
      y = e.MarginBounds.Y;
      e.Graphics.DrawString("Pizza Order Number " + orderNumber.ToString().PadLeft(4, '0'), myFont, Brushes.Black, e.MarginBounds.X, y);
      y += Convert.ToInt32(2 * myFont.GetHeight(e.Graphics));
      myFont = new Font("Arial", 12, FontStyle.Regular);
      e.Graphics.DrawString(txtFirstName.Text + " " + txtLastName.Text, myFont, Brushes.Black, e.MarginBounds.X, y);
      y += Convert.ToInt32(myFont.GetHeight(e.Graphics));
      e.Graphics.DrawString(txtAddress.Text, myFont, Brushes.Black, e.MarginBounds.X, y);
      y += Convert.ToInt32(myFont.GetHeight(e.Graphics));
      e.Graphics.DrawString(txtCity.Text + ", " + txtState.Text + " " + txtZip.Text, myFont, Brushes.Black, e.MarginBounds.X, y);
      y += Convert.ToInt32(myFont.GetHeight(e.Graphics));
      e.Graphics.DrawString(txtPhone.Text, myFont, Brushes.Black, e.MarginBounds.X, y);
      y += Convert.ToInt32(2 * myFont.GetHeight(e.Graphics));
      e.Graphics.DrawString("Ordered " + numberPizzas.ToString() + " Pizza(s):", myFont, Brushes.Black, e.MarginBounds.X, y);
      DataRow[] foundRows;
      foundRows = pizzasTable.Select("OrderNumber ='" + orderNumber.ToString() + "'");
      for (int i = 0; i < foundRows.Length; i++)
      {
        y += Convert.ToInt32(2 * myFont.GetHeight(e.Graphics));
        e.Graphics.DrawString(rdoSize[Convert.ToInt32(foundRows[i]["PizzaSize"])].Text + " Pizza", myFont, Brushes.Black, e.MarginBounds.X, y);
        y += Convert.ToInt32(myFont.GetHeight(e.Graphics));
        if (Convert.ToInt32(foundRows[i]["Delivery"]) == 1)
          e.Graphics.DrawString("Delivered", myFont, Brushes.Black, e.MarginBounds.X, y);
        else
          e.Graphics.DrawString("To be picked up", myFont, Brushes.Black, e.MarginBounds.X, y);
        s = foundRows[i]["Toppings"].ToString();
        for (int j = 0; j < numberToppings; j++)
        {
          if (s.Substring(j, 1).Equals("1"))
          {
            y += Convert.ToInt32(myFont.GetHeight(e.Graphics));
            e.Graphics.DrawString(chkTop[j].Text, myFont, Brushes.Black, e.MarginBounds.X, y);
          }
        }
        y += Convert.ToInt32(myFont.GetHeight(e.Graphics));
        e.Graphics.DrawString("Cost: $" + String.Format("{0:f2}", foundRows[i]["Cost"]), myFont, Brushes.Black, e.MarginBounds.X, y);
      }
      y += Convert.ToInt32(2 * myFont.GetHeight(e.Graphics));
      e.Graphics.DrawString("Total Cost: $" + String.Format("{0:f2}", totalCost), myFont, Brushes.Black, e.MarginBounds.X, y);
      e.HasMorePages = false;
    }

    private void txtPhone_KeyPress(object sender, KeyPressEventArgs e)
    {
      if ((int)e.KeyChar == 13)
        btnFind.PerformClick();
    }

    private void rdoSize_CheckedChanged(object sender, EventArgs e)
    {
      RadioButton buttonChecked = (RadioButton)sender;
      switch (buttonChecked.Name)
      {
        case "rdoSmall":
          pizzaSize = 0;
          break;
        case "rdoMedium":
          pizzaSize = 1;
          break;
        case "rdoLarge":
          pizzaSize = 2;
          break;
      }
    }

    private void btnSave_Click(object sender, EventArgs e)
    {
      btnSave.Visible = false;
      btnCancel.Visible = false;
      customersManager.EndCurrentEdit();
      DataRow currentRow = customersTable.Rows[customersTable.Rows.Count - 1];
      currentRow["PhoneNumber"] = txtPhone.Text;
      btnFind.Enabled = false;
      grpSize.Enabled = true;
      grpDeliver.Enabled = true;
      grpToppings.Enabled = true;
      txtSpecial.Enabled = true;
      btnBuild.Enabled = true;
      btnComplete.Enabled = false;
      txtPhone.Enabled = false;
      totalCost = 0;
      numberPizzas = 0;
    }

    private void btnCancel_Click(object sender, EventArgs e)
    {
      customersManager.CancelCurrentEdit();
      btnSave.Visible = false;
      btnCancel.Visible = false;
      btnFind.Enabled = true;
      txtFirstName.Visible = false;
      txtLastName.Visible = false;
      txtAddress.Visible = false;
      txtCity.Visible = false;
      txtState.Visible = false;
      txtZip.Visible = false;
      txtComment.Visible = false;
      txtPhone.Enabled = true;
      txtPhone.Text = "";
      txtPhone.Focus();
    }



  }
}