using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Data.OleDb;
using System.Drawing.Printing;


namespace Student_Identification
{
  public partial class frmStudentID : Form
  {
    public frmStudentID()
    {
      InitializeComponent();
    }

    OleDbConnection studentIDConnection;
    OleDbCommand studentIDCommand;
    OleDbDataAdapter studentIDAdapter;
    DataTable studentIDTable;
    CurrencyManager studentIDManager;
    string myState;
    int myBookmark;

    private void frmStudentID_Load(object sender, EventArgs e)
    {
      // connect to sales database
      studentIDConnection = new OleDbConnection("Provider=Microsoft.Jet.OLEDB.4.0; Data Source = " + Application.StartupPath + "\\..\\..\\student.mdb");
      studentIDConnection.Open();
      // establish studentID command object
      studentIDCommand = new OleDbCommand("SELECT * FROM Students ORDER BY LastName", studentIDConnection);
      // establish studentID data adapter/data table
      studentIDAdapter = new OleDbDataAdapter();
      studentIDAdapter.SelectCommand = studentIDCommand;
      studentIDTable = new DataTable();
      studentIDAdapter.Fill(studentIDTable);
      // bind controls
      txtFirstName.DataBindings.Add("Text", studentIDTable, "FirstName");
      txtLastName.DataBindings.Add("Text", studentIDTable, "LastName");
      lblSex.DataBindings.Add("Text", studentIDTable, "Sex");
      lblGrade.DataBindings.Add("Text", studentIDTable, "Grade");
      dtpBirthdate.DataBindings.Add("Text", studentIDTable, "Birthdate");
      lblPhotoFile.DataBindings.Add("Text", studentIDTable, "PictureFile");
      // establish currency manager
      studentIDManager = (CurrencyManager)this.BindingContext[studentIDTable];
      // show initial photo
      ShowUnbound();
      SetState("View");
    }

    private void frmStudentID_FormClosing(object sender, FormClosingEventArgs e)
    {
      if (myState.Equals("Edit") || myState.Equals("Add"))
      {
        MessageBox.Show("You must finish the current edit before stopping.", "", MessageBoxButtons.OK, MessageBoxIcon.Information);
        e.Cancel = true;
      }
      else
      {
        try
        {
          // save the update studentID table
          OleDbCommandBuilder studentIDAdapterCommands = new OleDbCommandBuilder(studentIDAdapter);
          studentIDAdapter.Update(studentIDTable);
        }
        catch (Exception ex)
        {
          MessageBox.Show(ex.Message, "Error Saving Database", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
        // close the connection
        studentIDConnection.Close();
        // dispose of the objects
        studentIDCommand.Dispose();
        studentIDAdapter.Dispose();
        studentIDTable.Dispose();
      }
    }

    private void ShowPhoto()
    {
      // display photo
      if (!lblPhotoFile.Text.Equals(""))
      {
        try
        {
          picItem.Image = Image.FromFile(lblPhotoFile.Text);
        }
        catch (Exception ex)
        {
          MessageBox.Show(ex.Message, "Error Loading Photo", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
      }
      else
      {
        picItem.Image = null;
      }
    }

    private void btnFirst_Click(object sender, EventArgs e)
    {
      studentIDManager.Position = 0;
      ShowUnbound();
    }

    private void btnPrevious_Click(object sender, EventArgs e)
    {
      studentIDManager.Position--;
      ShowUnbound();
    }

    private void btnNext_Click(object sender, EventArgs e)
    {
      studentIDManager.Position++;
      ShowUnbound();
    }

    private void btnLast_Click(object sender, EventArgs e)
    {
      studentIDManager.Position = studentIDManager.Count - 1;
      ShowUnbound();
    }

    private void ShowUnbound()
    {
      // sex choice
      if (lblSex.Text.Equals("M"))
        rdoMale.Checked = true;
      else
        rdoFemale.Checked = true;
      // grade choice
      switch (lblGrade.Text)
      {
        case "1":
          rdoFirst.Checked = true;
          break;
        case "2":
          rdoSecond.Checked = true;
          break;
        case "3":
          rdoThird.Checked = true;
          break;
        case "4":
          rdoFourth.Checked = true;
          break;
        case "5":
          rdoFifth.Checked = true;
          break;
        case "6":
          rdoSixth.Checked = true;
          break;
    }
      ShowPhoto();
    }

    private void btnExit_Click(object sender, EventArgs e)
    {
      this.Close();
    }

    private void SetState(string appState)
    {
      myState = appState;
      switch (myState)
      {
        case "View":
          btnFirst.Enabled = true;
          btnPrevious.Enabled = true;
          btnNext.Enabled = true;
          btnLast.Enabled = true;
          btnEdit.Enabled = true;
          btnSave.Enabled = false;
          btnCancel.Enabled = false;
          btnAdd.Enabled = true;
          btnDelete.Enabled = true;
          btnPrint.Enabled = true;
          btnExit.Enabled = true;
          txtFirstName.ReadOnly = true;
          txtLastName.ReadOnly = true;
          dtpBirthdate.Enabled = false;
          btnLoadPhoto.Enabled = false;
          grpSex.Enabled = false;
          grpGrade.Enabled = false;
          break;
        default: // "Edit", "Add"
          btnFirst.Enabled = false;
          btnPrevious.Enabled = false;
          btnNext.Enabled = false;
          btnLast.Enabled = false;
          btnEdit.Enabled = false;
          btnSave.Enabled = true;
          btnCancel.Enabled = true;
          btnAdd.Enabled = false;
          btnDelete.Enabled = false;
          btnPrint.Enabled = false;
          btnExit.Enabled = false;
          txtFirstName.ReadOnly = false;
          txtLastName.ReadOnly = false;
          dtpBirthdate.Enabled = true;
          btnLoadPhoto.Enabled = true;
          grpSex.Enabled = true;
          grpGrade.Enabled = true;
          break;
      }
      txtFirstName.Focus();
    }

    private void rdoSex_CheckedChanged(object sender, EventArgs e)
    {
      RadioButton buttonChecked = (RadioButton)sender;
      if (buttonChecked.Text.Equals("Male"))
        lblSex.Text = "M";
      else
        lblSex.Text = "F";
    }

    private void rdoGrade_CheckedChanged(object sender, EventArgs e)
    {
      RadioButton buttonChecked = (RadioButton)sender;
      switch (buttonChecked.Text)
      {
        case "First":
          lblGrade.Text = "1";
          break;
        case "Second":
          lblGrade.Text = "2";
          break;
        case "Third":
          lblGrade.Text = "3";
          break;
        case "Fourth":
          lblGrade.Text = "4";
          break;
        case "Fifth":
          lblGrade.Text = "5";
          break;
        case "Sixth":
          lblGrade.Text = "6";
          break;
      }
    }

    private void btnEdit_Click(object sender, EventArgs e)
    {
      SetState("Edit");
    }

    private void btnSave_Click(object sender, EventArgs e)
    {
      // Check for last name
      if (txtLastName.Text.Trim().Equals(""))
      {
        MessageBox.Show("You must enter a last name.", "Input Error", MessageBoxButtons.OK, MessageBoxIcon.Information);
        txtLastName.Focus();
        return;
      }
      string savedItem = txtLastName.Text;
      int savedRow;
      studentIDManager.EndCurrentEdit();
      studentIDTable.DefaultView.Sort = "LastName";
      savedRow = studentIDTable.DefaultView.Find(savedItem);
      studentIDManager.Position = savedRow;
      ShowUnbound();
      SetState("View");
    }

    private void btnCancel_Click(object sender, EventArgs e)
    {
      studentIDManager.CancelCurrentEdit();
      if (myState.Equals("Add"))
        studentIDManager.Position = myBookmark;
      ShowUnbound();
      SetState("View");
    }

    private void btnLoadPhoto_Click(object sender, EventArgs e)
    {
      try
      {
        if (dlgOpen.ShowDialog() == DialogResult.OK)
        {
          lblPhotoFile.Text = dlgOpen.FileName;
          ShowPhoto();
        }
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, "Error Opening Photo", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }

    private void btnAdd_Click(object sender, EventArgs e)
    {
      myBookmark = studentIDManager.Position;
      SetState("Add");
      studentIDManager.AddNew();
      // set unbound properties
      rdoMale.Checked = true; lblSex.Text = "M";
      rdoFirst.Checked = true; lblGrade.Text = "1";
      picItem.Image = null;
      lblPhotoFile.Text = "";
    }

    private void btnDelete_Click(object sender, EventArgs e)
    {
      if (MessageBox.Show("Are you sure you want to delete this record?", "Delete", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
      {
        studentIDManager.RemoveAt(studentIDManager.Position);
        ShowUnbound();
      }
      SetState("View");
    }

    private void txtFirstName_KeyPress(object sender, KeyPressEventArgs e)
    {
      if ((int)e.KeyChar == 13)
        txtLastName.Focus();
    }

    private void btnPrint_Click(object sender, EventArgs e)
    {
      // Declare the document
      PrintDocument studentIDDocument;
      // Create the document and name it
      studentIDDocument = new PrintDocument();
      studentIDDocument.DocumentName = "StudentID";
      // Add code handler
      studentIDDocument.PrintPage += new PrintPageEventHandler(this.PrintStudentID);
      // Print document in preview control
      int savedPosition = studentIDManager.Position;
      dlgPreview.Document = studentIDDocument;
      dlgPreview.ShowDialog();
      // Dispose of document when done printing
      studentIDDocument.Dispose();
      studentIDManager.Position = savedPosition;
    }

    private void PrintStudentID(object sender, PrintPageEventArgs e)
    {
      // draw border
      Pen myPen = new Pen(Color.Black, 3);
      e.Graphics.DrawRectangle(myPen, e.MarginBounds.X, e.MarginBounds.Y, 200, 350);
      Font myFont = new Font("Arial", 12, FontStyle.Bold);
      string s = "Your School Name";
      SizeF sSize = e.Graphics.MeasureString(s, myFont);
      e.Graphics.DrawString(s, myFont, Brushes.Black, e.MarginBounds.X + Convert.ToInt32(0.5 * (200 - sSize.Width)), e.MarginBounds.Y + 15);
      // position picture
      e.Graphics.DrawImage(picItem.Image, e.MarginBounds.X + 25, e.MarginBounds.Y + 50, 150, 200);
      // first/last name/grade
      myFont = new Font("Arial", 14);
      int y = e.MarginBounds.Y + 260;
      s = txtFirstName.Text;
      sSize = e.Graphics.MeasureString(s, myFont);
      e.Graphics.DrawString(s, myFont, Brushes.Black, e.MarginBounds.X + Convert.ToInt32(0.5 * (200 - sSize.Width)), y);
      y += Convert.ToInt32(myFont.GetHeight(e.Graphics));
      s = txtLastName.Text;
      sSize = e.Graphics.MeasureString(s, myFont);
      e.Graphics.DrawString(s, myFont, Brushes.Black, e.MarginBounds.X + Convert.ToInt32(0.5 * (200 - sSize.Width)), y);
      y += Convert.ToInt32(myFont.GetHeight(e.Graphics));
      s = "Grade " + lblGrade.Text;
      sSize = e.Graphics.MeasureString(s, myFont);
      e.Graphics.DrawString(s, myFont, Brushes.Black, e.MarginBounds.X + Convert.ToInt32(0.5 * (200 - sSize.Width)), y);
      e.HasMorePages = false;
    }



  }
}