using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using ADOX;
using System.Data.OleDb;
using System.Drawing.Printing;

namespace Weight_Monitor
{
  public partial class frmWeightMonitor : Form
  {
    public frmWeightMonitor()
    {
      InitializeComponent();
    }

    OleDbConnection weightConnection;
    OleDbCommand weightCommand;
    OleDbDataAdapter weightAdapter;
    DataTable weightTable;
    int plotRange = 0;
    int BMIHeight, BMIWeight;
    bool metric;
    string[] monthNames = { "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December" };


    private void frmWeightMonitor_Load(object sender, EventArgs e)
    {
      for (int y = 1900; y <= 2100; y++)
        cboYear.Items.Add(y.ToString());
      cboYear.Text = calDate.SelectionStart.Year.ToString();
      tabWeight.SelectedTab = tabPageData;
      grdWeight.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;
      grdWeight.AllowUserToAddRows = false;
      rdoPounds.Checked = true;
    }

    private void tabWeight_SelectedIndexChanged(object sender, EventArgs e)
    {
      switch (tabWeight.SelectedIndex)
      {
        case (1):
          WeightData();
          break;
        case (2):
          ComputeBMI();
          break;
      }
    }
    private void ComputeBMI()
    {
      float BMI, w, h;
      if (metric)
      {
        w = (float) nudWeight.Value;
        h = (float) nudHeight.Value / 100;
      }
      else
      {
        w = ((float) nudWeight.Value) * 0.4536F;
        h = ((float) nudHeight.Value) * (2.54F / 100);
      }
      if (h != 0)
        BMI = w / (h * h);
      else
        BMI = 0;
      lblBMI.Text = String.Format("{0:f1}", BMI);
      if (BMI > 27)
        lblBMI.BackColor = lblOverRisk.BackColor;
      else if (BMI > 25 && BMI <= 27)
        lblBMI.BackColor = lblOver.BackColor;
      else if (BMI >= 20 && BMI <= 25)
        lblBMI.BackColor = lblHealthy.BackColor;
      else
        lblBMI.BackColor = lblUnder.BackColor;

      if (BMI > 30)
        lblLine.Top = lblOverRisk.Top;
      else if (BMI < 18)
        lblLine.Top = lblUnder.Top + lblUnder.Height;
      else
        lblLine.Top = Convert.ToInt32(lblOverRisk.Top + (30 - BMI) * (lblOverRisk.Height + lblOver.Height + lblHealthy.Height + lblUnder.Height) / 12);
      lblLine.Top -= 2;
      lblBMI.Top = Convert.ToInt32(lblLine.Top - 0.5 * lblBMI.Height);
      lblBMITitle.Top = lblBMI.Top - lblBMITitle.Height;

    }

    private void rdoUnits_CheckedChanged(object sender, EventArgs e)
    {
      RadioButton whichButton = (RadioButton) sender;
      nudWeight.Minimum = 1; nudWeight.Maximum = 1000;
      nudHeight.Minimum = 1; nudHeight.Maximum = 1000;
      switch (whichButton.Name)
      {
        case ("rdoPounds"):
          metric = false;
          lblWeightHeading.Text = "Weight (pounds)";
          lblHeightHeading.Text = "Height (inches)";
          BMIWeight = 140;
          BMIHeight = 66;
          nudWeight.Minimum = 80; nudWeight.Maximum = 300;
          nudHeight.Minimum = 48; nudHeight.Maximum = 84;
          break;
        case ("rdoKilos"):
          metric = true;
          lblWeightHeading.Text = "Weight (kilograms)";
          lblHeightHeading.Text = "Height (centimeters)";
          BMIWeight = 64;
          BMIHeight = 168;
          nudWeight.Minimum = 36; nudWeight.Maximum = 136;
          nudHeight.Minimum = 120; nudHeight.Maximum = 215;
          break;
      }
      nudWeight.Value = BMIWeight;
      nudHeight.Value = BMIHeight;
      ComputeBMI();
    }

    private void nudWeight_ValueChanged(object sender, EventArgs e)
    {
      ComputeBMI();
    }

    private void nudHeight_ValueChanged(object sender, EventArgs e)
    {
      ComputeBMI();
    }

    private void btnNew_Click(object sender, EventArgs e)
    {
      Catalog weightDatabase = new Catalog();
      Table databaseTable;
      // close any open database
      CloseConnection();
      try
      {
        // get filename
        if (dlgSave.ShowDialog() == DialogResult.OK)
        {
          weightDatabase.Create("Provider=Microsoft.Jet.OLEDB.4.0; Data Source = " + dlgSave.FileName);
          // create table
          databaseTable = new Table();
          databaseTable.Name = "Weight";
          // add fields
          databaseTable.Columns.Append("WeightDate", DataTypeEnum.adDate, 20);
          databaseTable.Columns.Append("Weight", DataTypeEnum.adSingle, 10);
          databaseTable.Columns.Append("Comment", DataTypeEnum.adWChar, 50);
          databaseTable.Columns["Weight"].Attributes = ColumnAttributesEnum.adColNullable;
          databaseTable.Columns["Comment"].Attributes = ColumnAttributesEnum.adColNullable;
          // primary key
          databaseTable.Keys.Append("PK_Weight", KeyTypeEnum.adKeyPrimary, "WeightDate", null, null);
          databaseTable.Indexes.Append("WeightDate", "WeightDate");
          weightDatabase.Tables.Append(databaseTable);
          // connect to database
          OpenConnection(dlgSave.FileName);
          // fill dates
          DateTime tableDate = new DateTime(Convert.ToInt32(cboYear.Text), 1, 1);
          DateTime nextYear = new DateTime(Convert.ToInt32(cboYear.Text) + 1, 1, 1);
          int nDays = (nextYear - tableDate).Days;
          DataRow newRow;
          for (int n = 1; n <= nDays; n++)
          {
            newRow = weightTable.NewRow();
            newRow["WeightDate"] = tableDate;
            weightTable.Rows.Add(newRow);
            tableDate += new TimeSpan(1, 0, 0, 0);
          }
          grdWeight_CellClick(null, null);
        }
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, "Error Creating Database", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
      finally
      {
        weightDatabase = null;
      }
    }

    private void OpenConnection(string fName)
    {
      try
      {
        weightConnection = new OleDbConnection("Provider=Microsoft.Jet.OLEDB.4.0; Data Source = " + fName);
        weightConnection.Open();
        // establish command object
        weightCommand = new OleDbCommand("SELECT * FROM Weight ORDER BY WeightDate", weightConnection);
        // establish data adapter/data table
        weightAdapter = new OleDbDataAdapter();
        weightAdapter.SelectCommand = weightCommand;
        weightTable = new DataTable();
        weightAdapter.Fill(weightTable);
        grdWeight.DataSource = weightTable;
        grdWeight.Columns[0].ReadOnly = true;
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, "Error Opening Database", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }

    private void CloseConnection()
    {
      try
      {
        if (!(weightConnection == null))
        {
          if (weightConnection.State == ConnectionState.Open)
          {
            OleDbCommandBuilder weightAdapterCommand = new OleDbCommandBuilder(weightAdapter);
            weightAdapter.Update(weightTable);
            weightConnection.Close();
            weightConnection.Dispose();
            weightCommand.Dispose();
            weightAdapter.Dispose();
            weightTable.Dispose();
            weightAdapterCommand.Dispose();
          }
        }
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, "Error Saving Database", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }

    private void frmWeightMonitor_FormClosing(object sender, FormClosingEventArgs e)
    {
      // close connection
      CloseConnection();
    }

    private void grdWeight_EditingControlShowing(object sender, DataGridViewEditingControlShowingEventArgs e)
    {
      this.grdWeight.EditingControl.KeyPress -= new KeyPressEventHandler(this.grdWeight_KeyPress);
      this.grdWeight.EditingControl.KeyPress += new KeyPressEventHandler(this.grdWeight_KeyPress);
    }

    private void btnOpen_Click(object sender, EventArgs e)
    {
      try
      {
        // close connection if open
        CloseConnection();
        // get filename
        if (dlgOpen.ShowDialog() == DialogResult.OK)
        {
          // connect to database
          OpenConnection(dlgOpen.FileName);
          grdWeight_CellClick(null, null);
        }
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, "Error Opening File", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }

    private void grdWeight_CellClick(object sender, DataGridViewCellEventArgs e)
    {
      calDate.SelectionStart = Convert.ToDateTime(grdWeight[0, grdWeight.CurrentCell.RowIndex].Value);
    }

    private void grdWeight_KeyPress(object sender, KeyPressEventArgs e)
    {
      // numeric entries only in first columns
      int currentColumn = grdWeight.CurrentCell.ColumnIndex;
      if (currentColumn == 1)
      {
        if ((e.KeyChar >= '0' && e.KeyChar <= '9') || e.KeyChar == '-' || e.KeyChar == '.' || (int)e.KeyChar == 8)
          e.Handled = false;
        else
          e.Handled = true;
      }
    }

    private void calDate_DateChanged(object sender, DateRangeEventArgs e)
    {
      // match selected date to grid (if possible)
      if (!(weightConnection == null))
      {
        weightTable.DefaultView.Sort = "WeightDate";
        int dateRow = weightTable.DefaultView.Find(calDate.SelectionStart);
        if (dateRow != -1)
          grdWeight.CurrentCell = grdWeight[1, dateRow];
      }
    }

    private void WeightData()
    {
      // make sure there is a data table
      if (weightConnection == null)
        return;
      float temp;
      int xStart = 0, xEnd = 0, nDays, isLeap;
      float wH, wL;
      float sumW;
      int nW;
      double sx, sx2, sxy;
      float a1;
      bool noData;
      float[] weight = new float[366];
      if (weightTable.Rows.Count == 365)
        isLeap = 0;
      else
        isLeap = 1;
      switch (plotRange)
      {
        case 0:
          xStart = 0;
          xEnd = 30;
          break;
        case 1:
          xStart = 31;
          xEnd = 58 + isLeap;
          break;
        case 2:
          xStart = 59 + isLeap;
          xEnd = 89 + isLeap;
          break;
        case 3:
          xStart = 90 + isLeap;
          xEnd = 119 + isLeap;
          break;
        case 4:
          xStart = 120 + isLeap;
          xEnd = 150 + isLeap;
          break;
        case 5:
          xStart = 151 + isLeap;
          xEnd = 180 + isLeap;
          break;
        case 6:
          xStart = 181 + isLeap;
          xEnd = 211 + isLeap;
          break;
        case 7:
          xStart = 212 + isLeap;
          xEnd = 242 + isLeap;
          break;
        case 8:
          xStart = 243 + isLeap;
          xEnd = 272 + isLeap;
          break;
        case 9:
          xStart = 273 + isLeap;
          xEnd = 303 + isLeap;
          break;
        case 10:
          xStart = 304 + isLeap;
          xEnd = 333 + isLeap;
          break;
        case 11:
          xStart = 334 + isLeap;
          xEnd = 364 + isLeap;
          break;
      }
      nDays = xEnd - xStart + 1;
      // temperature data
      wH = -1000; wL = 1000;
      sumW = 0; nW = 0;
      sx = 0; sx2 = 0; sxy = 0;
      noData = true;
      for (int x = xStart; x <= xEnd; x++)
      {
        if (weightTable.Rows[x]["Weight"] != DBNull.Value)
        {
          temp = Convert.ToSingle(weightTable.Rows[x]["Weight"]);
          noData = false;
          weight[x - xStart] = temp;
          if (temp > wH)
            wH = temp;
          if (temp < wL)
            wL = temp;
          sumW += temp;
          nW++;
          sx += x;
          sx2 += x * x;
          sxy += x * temp;
        }
        else
        {
          weight[x - xStart] = -1000.0F;
        }
      }
      if (!noData)
      {
        lblHigh.Text = String.Format("{0:f1}", wH);
        lblLow.Text = String.Format("{0:f1}", wL);
        if (nW > 1)
        {
          a1 = Convert.ToSingle((nW * sxy - sx * sumW) / (nW * sx2 - sx * sx));
          if (a1 >= 0)
            lblTrend.Text = "+" + String.Format("{0:f1}", 7 * a1) + " Per Week";
          else
            lblTrend.Text = String.Format("{0:f1}", 7 * a1) + " Per Week";
        }
        lblAve.Text = String.Format("{0:f1}", sumW / nW);
      }
      else
      {
        lblHigh.Text = "---";
        lblLow.Text = "---";
        lblAve.Text = "---";
        lblTrend.Text = "---";
      }
      WeightPlot weightPlot = new WeightPlot(pnlWeightPlot);
      if (!noData)
        weightPlot.Draw(nDays, weight);
    }

    private void rdoMonth_CheckedChanged(object sender, EventArgs e)
    {
      RadioButton whichButton = (RadioButton)sender;
      switch (whichButton.Name)
      {
        case "rdoJanuary":
          plotRange = 0;
          break;
        case "rdoFebruary":
          plotRange = 1;
          break;
        case "rdoMarch":
          plotRange = 2;
          break;
        case "rdoApril":
          plotRange = 3;
          break;
        case "rdoMay":
          plotRange = 4;
          break;
        case "rdoJune":
          plotRange = 5;
          break;
        case "rdoJuly":
          plotRange = 6;
          break;
        case "rdoAugust":
          plotRange = 7;
          break;
        case "rdoSeptember":
          plotRange = 8;
          break;
        case "rdoOctober":
          plotRange = 9;
          break;
        case "rdoNovember":
          plotRange = 10;
          break;
        case "rdoDecember":
          plotRange = 11;
          break;
      }
      WeightData();
    }

    private void btnExit_Click(object sender, EventArgs e)
    {
      this.Close();
    }

    private void btnPrint_Click(object sender, EventArgs e)
    {
      // make sure there is a data table
      if (weightConnection == null)
        return;
      // Declare the document
      PrintDocument recordDocument;
      // Create the document and name it
      recordDocument = new PrintDocument();
      recordDocument.DocumentName = "Weight Data";
      recordDocument.PrintPage += new PrintPageEventHandler(this.PrintWeightReport);
      // Preview document
      dlgPreview.Document = recordDocument;
      dlgPreview.ShowDialog();
      // Dispose of document when done printing
      recordDocument.Dispose();
    }

    private void PrintWeightReport(object sender, PrintPageEventArgs e)
    {
      Font myFont = new Font("Courier New", 12, FontStyle.Regular);
      int y = 125;
      e.Graphics.DrawString("Weight Monitor", myFont, Brushes.Black, e.MarginBounds.X, y);
      y += Convert.ToInt32(2 * myFont.GetHeight(e.Graphics));
      myFont = new Font("Courier New", 12, FontStyle.Regular | FontStyle.Underline);
      e.Graphics.DrawString("Weight Summary for " + monthNames[plotRange], myFont, Brushes.Black, e.MarginBounds.X, y);
      y += Convert.ToInt32(2 * myFont.GetHeight(e.Graphics));
      myFont = new Font("Courier New", 12, FontStyle.Regular);
      e.Graphics.DrawString("  High  = " + lblHigh.Text, myFont, Brushes.Black, 150, y);
      y += Convert.ToInt32(myFont.GetHeight(e.Graphics));
      e.Graphics.DrawString("  Ave   = " + lblAve.Text, myFont, Brushes.Black, 150, y);
      y += Convert.ToInt32(myFont.GetHeight(e.Graphics));
      e.Graphics.DrawString("  Low   = " + lblLow.Text, myFont, Brushes.Black, 150, y);
      y += Convert.ToInt32(myFont.GetHeight(e.Graphics));
      e.Graphics.DrawString("  Trend = " + lblTrend.Text, myFont, Brushes.Black, 150, y);
      if (pnlWeightPlot.BackgroundImage != null)
        e.Graphics.DrawImage(pnlWeightPlot.BackgroundImage, 150, 300, 500, Convert.ToInt32(500 * pnlWeightPlot.ClientSize.Height / pnlWeightPlot.ClientSize.Height));
      e.HasMorePages = false;
    }



  }
}